<?php
/**
* RDF Generator
*
* Class to generate RDF Channels (RDF Version 1.0)
*
* (c) 2002 Stefan Fischer
*
* @author  Stefan Fischer <post@stefan-fischer.net>
* @version 0.1
* @package RDFGenerator
* @access  public
*/

class RDFGenerator
{

/**
* XML encoding
* @var string
*/
 var $encoding = 'UTF-8';

/**
* XML version
* @var string
*/
 var $version = '1.0';

/**
* XML namespaces
* @var array
*/
 var $namespaces = array('RDF'	=> 'http://www.w3.org/1999/02/22-rdf-syntax-ns#',
 						 'XML'	=> 'http://purl.org/rss/1.0/',
 						 'DC'	=> 'http://purl.org/dc/elements/1.1/'
 						 );

/**
* RDF channel
* @var array
*/
 var $channel = array();

/**
* RDF items
* @var array
*/
 var $items = array();

/**
* RDF image
* @var array
*/
 var $image = array();

/**
* RDF textinput
* @var array
*/
 var $input = array();

/**
* check if DC Dublin Core metadata are used
* @var boolean
*/
 var $checkdc = false;


/**
* prints out generated channel in RDF-Format
*
* @param  void
* @access public
*/
function printRDF()  {

	header('Content-Type: text/xml');
	print $this->_buildRDF();
}


/**
* writes channel to file
*
* @param  string  $file   path to file
* @access public
*/
function writeRDF($file)  {

	$data = $this->_buildRDF();

	if (!$fh = fopen($file, 'w')) {
		$this->error('Could not open file <b>'.$file.'</b>');
	} else {
		flock($fh,2);
		fwrite($fh, $data);
		flock($fh,3);
		fclose($fh);
	}
}


/**
* creates new channel and adds basic channel data
*
* @param  array   $ch  channel data
* @access public
*/
function createChannel($ch)  {

	$must = array('title', 'link');
	$temp = $this->checkData($ch, $must, 'new Channel');

	$this->channel = $temp;

	if ( isset($temp['dc']) && !empty($temp['dc']) )  {
		$this->checkdc = true;
	}

}


/**
* adds image to channel
*
* @param  string   $img  image data
* @access public
*/
function addImage($img)  {

	$must = array('title', 'link', 'url');
	$temp = $this->checkData($img, $must, 'Image');

	$this->image = $temp;
}


/**
* adds textinput form to channel
*
* @param  string   $in  form data
* @access public
*/
function addTextinput($in)  {

	$must = array('title', 'link', 'name');
	$temp = $this->checkData($in, $must, 'Textinput form');

	$this->input = $temp;
}


/**
* adds item to channel
*
* @param  string   $item  item data
* @access public
*/
function addItem($item)  {

    $nr = count($this->items)+1;

	$must = array('Title', 'link');
	$temp = $this->checkData($item, $must, 'Item '.$nr.'');

	if ( isset($temp['dc']) && !empty($temp['dc']) )  {
		$this->checkdc = true;
	}

	array_push($this->items, $temp);
}


/**
* sets XML version
*
* @param  string   $str  XML version
* @access public
*/
function setVersion($str)  {

	$this->version = $str;
}


/**
* sets XML encoding
*
* @param  string   $str  XML encoding
* @access public
*/
function setEncoding($str)  {

	$this->encoding = $str;
}


/**
* set namespaces
*
* @param   array   $ns  namespaces
* @access  public
*/
function setNamespaces($ns)  {

	for (reset($ns); list($k, $v) = each($ns);) {
		if (isset($this->namespaces[$k])) {
			$this->namespaces[$k] = $v;
		}  else  {
			$this->error('Unknown Namespace <b>'.$k.'</b>');
		}
	}
}


/**
* generates channel
*
* @param  void
* @access private
*/
function _buildRDF()  {

	$rdf  = '<?xml version="'.$this->version.'" encoding="'.$this->encoding.'" ?>';
	$rdf .= '<rdf:RDF ';
	$rdf .= 'xmlns:rdf="'.$this->namespaces['RDF'].'" ';
	$rdf .= 'xmlns="'.$this->namespaces['XML'].'" ';

	if($this->checkdc)
		$rdf .= 'xmlns:dc="'.$this->namespaces['DC'].'"';

	$rdf .= '>';

    $rdf .= $this->_buildChannel();
    $rdf .= $this->_buildImage();
    $rdf .= $this->_buildItems();
    $rdf .= $this->_buildInput();

    $rdf .= '</rdf:RDF>';

	return $rdf;
}


/**
* generates basic channel data
*
* @param  void
* @access private
*/
function _buildChannel() {

	$channel  = '<channel rdf:about="'.$this->channel['link'].'">';

	for (reset($this->channel); list($k, $v) = each($this->channel);)  {
		if ( strtoupper($k) != 'DC' )  {
			$channel .= '<'.$k.'>'.htmlspecialchars($v).'</'.$k.'>';
		}
    }

	if (isset($this->image) && !empty($this->image)) {
		$channel .= '<image rdf:resource="'.$this->image['url'].'"/>';
	}

	if (isset($this->input) && !empty($this->input)) {
		$channel .= '<textinput rdf:resource="'.$this->input['link'].'"/>';
	}

	if (isset($this->items) && !empty($this->items)) {

		$channel .= '<items><rdf:Seq>';

			for ($i = 0; $i < count($this->items); $i++) {
				$channel .= '<rdf:li resource="'.$this->items[$i]['link'].'"/>';
			}

		$channel .= '</rdf:Seq></items>';
	}

	if ( isset($this->channel['dc']) && !empty($this->channel['dc']) )  {

		for (reset($this->channel['dc']); list($k, $v) = each($this->channel['dc']);)  {

			$channel .= '<dc:'.$k.'>'.htmlspecialchars($v).'</dc:'.$k.'>';

		}

	}

	$channel .= '</channel>';
	return $channel;
}


/**
* generates image data
*
* @param  void
* @access private
*/
function _buildImage() {

	if ( isset($this->image) && !empty($this->image) ) {

		$image  = '<image rdf:about="'.$this->image['url'].'">';
		$image .= '<title>'.htmlspecialchars($this->image['title']).'</title>';
		$image .= '<link>'.$this->image['link'].'</link>';
		$image .= '<url>'.$this->image['url'].'</url>';
		$image .= '</image>';

		return $image;

	} else {
		return '';
	}
}


/**
* generates items data
*
* @param  void
* @access private
*/
function _buildItems() {

	if (isset($this->items) && !empty($this->items)) {

    	$item = '';

		for ($i = 0; $i < count($this->items); $i++) {

			$item .= '<item rdf:about="'.$this->items[$i]['link'].'">';
			$item .= '<title>'.htmlspecialchars($this->items[$i]['title']).'</title>';
			$item .= '<link>'.$this->items[$i]['link'].'</link>';
			$item .= '<description>'.htmlspecialchars($this->items[$i]['description']).'</description>';

				if ( isset($this->items[$i]['dc']) && !empty($this->items[$i]['dc']) )  {

					for (reset($this->items[$i]['dc']); list($k, $v) = each($this->items[$i]['dc']);)  {
            			$item .= '<dc:'.$k.'>'.htmlspecialchars($v).'</dc:'.$k.'>';
               	 	}
           		}
			$item .= '</item>';
		}

		return $item;
	} else {
		return '';
	}
}


/**
* generates textinput data
*
* @param  void
* @access private
*/
function _buildInput() {

	if (isset($this->input) && !empty($this->input)) {

		$input  = '<textinput rdf:about="'.$this->input['link'].'">';
		$input .= '<title>'.htmlspecialchars($this->input['title']).'</title>';
		$input .= '<description>'.htmlspecialchars($this->input['description']).'</description>';
		$input .= '<name>'.htmlspecialchars($this->input['name']).'</name>';
		$input .= '<link>'.$this->input['link'].'</link>';
		$input .= '</textinput>';

  		return $input;

	}  else  {
       	return '';
	}
}


/**
* checks if all needed data is added to channel
*
* @param  array    $data    data of channel section
* @param  array    $needed  needed data for this section
* @param  string   $submsg  Errormessage
* @return array    $temp    checked channel data
* @access private
*/

function checkData($data, $needed = array(), $submsg = '')  {

	$temp = array();

	for (reset($data); list($k, $v) = each($data);)  {
		$temp[strtolower($k)] = $v;
	}

	for ($i = 0; $i < count($needed); $i++) {

		if ( !isset($temp[strtolower($needed[$i])]) )  {
			$this->error('<b>'.strtoupper($needed[$i]).'</b> for '.$submsg.' is missing');
		}
   }

	return $temp;
}


/**
* Errormessage
*
* @param  string   $msg  Errormessage
* @access private
*/
function error($msg)  {

	die(printf("ERROR: %s", $msg));
}

} // end of class

?>
